﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WSCF.RevCtrl.Test.IService;
using WSCF.RevCtrl.Test.Server.ServiceClient;

namespace WSCF.RevCtrl.Test.Server
{
    public class Chat : IChatService,WSCF.RevCtrl.IRevCtrlAdmin
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public int CustomerCount { get; private set; }

        public CustomerInfo[] CustomerInfos { get; private set; }
        


        public event EventHandler HavingNewMsgInGroupChat;

        List<CustomerContext> customerContexts = new List<CustomerContext>();
        List<string> msgs = new List<string>();

        public void GetRecentMsgs(AsyncCBHandler asyncDelegate, object asyncContext)
        {
            asyncDelegate?.Invoke(asyncContext, new GetRecentMsgsReponse()
            {
                Msgs = msgs.ToArray()
            });
        }

        public void SendMsgToGroupChat(string msg, AsyncCBHandler asyncDelegate, object asyncContext)
        {
            var cc = asyncContext as WSCF.CC;
            var customerContext = customerContexts.Find(c => c.revCtrl_Proxy == cc.proxy);
            if (customerContext == null) {
                //异常，居然之前没有记录
                throw new Exception("有人在群聊,居然之前没有记录");
            }

            string text = $"{DateTime.Now} [{customerContext.customer.Name}]: {msg}";
            msgs.Add(text);
            if (msgs.Count() > 100)
                msgs.RemoveAt(0);

            HavingNewMsgInGroupChat?.Invoke(this, new HavingNewMsgInGroupChatEventArgs()
            {
                msg = text
            });

            asyncDelegate?.Invoke(asyncContext, new SendMsgToGroupChatReponse());
        }

        public void SendMsgToPrivateChat(string targetGuid, string msg, AsyncCBHandler asyncDelegate, object asyncContext)
        {
            var cc = asyncContext as WSCF.CC;
            var customerContext = customerContexts.Find(c => c.revCtrl_Proxy == cc.proxy);
            if (customerContext == null)
            {
                //异常，居然之前没有记录
                throw new Exception("有人在私聊,居然之前没有记录");
            }

            var customerContext_target = customerContexts.Find(c => c.revCtrl_Proxy.ID == targetGuid);
            if (customerContext_target == null)
            {
                //没有这个目标
                asyncDelegate?.Invoke(asyncContext, new SendMsgToPrivateChatReponse() { isSuccess = false });
                return;
            }

            customerContext_target.customer.SendMsgToPrivateChat(
                customerContext.revCtrl_Proxy.ID,
                customerContext.customer.Name,
                DateTime.Now,
                msg);
            asyncDelegate?.Invoke(asyncContext, new SendMsgToPrivateChatReponse() { isSuccess = true });
        }

        public void OnOpen(RevCtrl_Proxy proxy)
        {
            //有人上线了
            var customerContext = customerContexts.Find(cc => cc.revCtrl_Proxy == proxy);
            if (customerContext == null)
            {
                //这是个新顾客

                //创键顾客服务的代理
                var customer = new CustomerServiceClient();
               
                customer.PropertyChanged += (s, e) =>
                {
                    if (e.PropertyName == nameof(customer.Name))
                    {
                        //名字被刷新

                        //刷新 CustomerInfos 列表
                        updateCustomerInfos();
                    }
                };
                proxy.AddClient("Customer", customer);


                //记录这个顾客的上下文
                customerContext = new CustomerContext()
                {
                    revCtrl_Proxy = proxy,
                    customer = customer
                };
                customerContexts.Add(customerContext);

                //刷新 CustomerInfos 列表
                updateCustomerInfos();
            }
            else {
                //他怎么又上线了，异常
                throw new Exception("他怎么又上线了，异常");
            }
        }

        /// <summary>
        /// 刷新 CustomerInfos 列表；
        /// customerContexts 转为 CustomerInfos
        /// </summary>
        void updateCustomerInfos() 
        {
            CustomerInfos = customerContexts.Select(cc =>
            {
                return new CustomerInfo()
                {
                    Guid = cc.revCtrl_Proxy.ID,
                    Name = cc.customer.Name
                };
            }).ToArray();
            CustomerCount = CustomerInfos.Count();
        }


        public void OnClose(RevCtrl_Proxy proxy)
        {
            //有人下线了
            var customerContext = customerContexts.Find(cc => cc.revCtrl_Proxy == proxy);
            if (customerContext == null)
            {
                //异常，居然之前没有记录
                throw new Exception("有人下线了,居然之前没有记录");
            }
            else
            {
                //删除顾客上下文
                customerContexts.Remove(customerContext);

                //刷新 CustomerInfos 列表
                updateCustomerInfos();
            }
        }

        public void OnDispose(RevCtrl_Proxy proxy)
        {
            OnClose(proxy);
        }
    }

    class CustomerContext 
    {
        public RevCtrl_Proxy revCtrl_Proxy;
        public ICustomerService customer;
    }
}
